"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Player = void 0;
var shapes_1 = require("./shapes");
var errors_1 = require("./errors");
var events_1 = require("./events");
var moves_1 = require("./moves");
var logger_1 = require("./logger");
var logger = (0, logger_1.default)("player.js");
var CardNotFoundError = (0, errors_1.createError)("CardNotFoundError");
var OutOfTurnError = (0, errors_1.createError)("OutOfTurnError");
var InvalidArgumentError = (0, errors_1.createError)("InvalidArgumentError");
var ExpectedToPickError = (0, errors_1.createError)("ExpectedToPickError");
var PlayValidationFailedError = (0, errors_1.createError)("PlayValidationFailedError");
var InvalidArgumentTypeError = (0, errors_1.createTypeError)("InvalidArgumentTypeError");
var CardNeededUndefinedError = (0, errors_1.createTypeError)("CardNeededUndefinedError");
/**
 * A player participates in a whot game, by holding and playing a card when it's his/her turn
 *
 * @event player:play when a card is played by being added to the pile
 * @event player:market when the player goes to market
 */
var Player = /** @class */ (function () {
    function Player(props) {
        var _this = this;
        this.cards = [];
        this.turn = false;
        this.hasWon = false;
        this.toPick = 0;
        this.emit = function () { return _this; };
        /**
         * checks that it is possible to play a card
         *
         * condition is true if:
         *   - card matches card at the top of the pile
         *   - top pile card is a Whot!, and is the first card in the game
         */
        this.validator = function (card) {
            return _this.pile().top().matches(card) || _this.pile().firstCardIsWhot();
        };
        this.add = function (_cards_) {
            if (Array.isArray(_cards_)) {
                _cards_.forEach(function (card) { return _this.cards.push(card); });
                _this.emitter.emit("player:add", _cards_);
                _this.emit("add", _cards_);
            }
            else {
                throw InvalidArgumentTypeError("_cards_", Array);
            }
        };
        this.hand = function () { return _this.cards.slice(0); };
        this.pick = function () {
            var marketCards = _this.market().pick(_this.toPick || 1);
            if (!Array.isArray(marketCards))
                throw InvalidArgumentTypeError("marketCards", Array);
            _this.add(marketCards);
            _this.emit("market", marketCards);
            _this.emitter.emit("player:market", _this, marketCards);
            _this.toPick = 0;
            return marketCards;
        };
        this.play = function (index, iNeed) {
            if (_this.turn) {
                var card = _this.cards[index];
                if (card) {
                    if (_this.toPick === 0 || card.shape === shapes_1.Shapes.Whot) {
                        if (card.shape === shapes_1.Shapes.Whot) {
                            if (!iNeed) {
                                throw CardNeededUndefinedError("player: ".concat(_this.id, ", card: ").concat(card.shape, ", iNeed: ").concat(iNeed), String);
                            }
                            else {
                                card.iNeed = iNeed;
                            }
                        }
                        if (_this.validator(card)) {
                            _this.cards.splice(index, 1);
                            _this.emit("play", card);
                            _this.emitter.emit("player:play", _this, card);
                            if (typeof _this.pile === "function") {
                                _this.pile().push([card]);
                            }
                            if (_this.empty()) {
                                if (_this.pile().top().move === moves_1.default.None) {
                                    _this.emitter.emit("player:checkup", _this);
                                    _this.emit("checkup", _this);
                                    _this.hasWon = true;
                                }
                                else {
                                    _this.pick();
                                }
                            }
                            if (_this.lastCard()) {
                                _this.emitter.emit("player:last-card", _this);
                                _this.emit("last-card", _this);
                            }
                            return card;
                        }
                        else {
                            throw PlayValidationFailedError(JSON.stringify({ top: _this.pile().top(), card: card }));
                        }
                    }
                    else {
                        throw ExpectedToPickError("player ".concat(_this.id, " is expected to pick ").concat(_this.toPick, " cards from market"));
                    }
                }
                else {
                    throw CardNotFoundError("no card found in player ".concat(_this.id, " pile at index ").concat(index));
                }
            }
            else {
                throw OutOfTurnError("player ".concat(_this.id, " is out of turn"));
            }
        };
        this.lastCard = function () { return _this.cards.length === 1; };
        this.empty = function () { return _this.cards.length === 0; };
        this.canPlay = function () {
            return _this.cards.findIndex(function (card) { return card.matches(_this.pile().top()); }) >= 0 &&
                _this.toPick === 0;
        };
        this.canMatchMove = function (move) {
            return _this.cards.findIndex(function (card) { return card.move === (move || _this.pile().top().move); }) >= 0;
        };
        this.render = function () {
            return "id: ".concat(_this.id, " count: ").concat(_this.cards.length, " hand: [").concat(_this.cards
                .map(function (card) { return card.value + (0, shapes_1.renderShape)(card.shape); })
                .join(","), "]");
        };
        if (!props)
            throw InvalidArgumentError("props");
        if (!props.id)
            throw InvalidArgumentError("props.id");
        if (!props.market || typeof props.market !== "function")
            throw InvalidArgumentError("props.market");
        if (!props.pile)
            logger.warn("No Pile Function supplied");
        else if (typeof props.pile !== "function")
            throw InvalidArgumentError("props.pile must be a function");
        if (!props.emitter) {
            logger.warn("No EventEmitter supplied");
            this.emitter = props.emitter = new events_1.EventEmitter();
        }
        else {
            this.emitter = props.emitter;
        }
        this.id = props.id;
        this.market = props.market;
        this.pile = props.pile;
        (0, events_1.eventify)(this);
    }
    return Player;
}());
exports.Player = Player;
exports.default = Player;
//# sourceMappingURL=player.js.map