"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Turn = void 0;
var errors_1 = require("./errors");
var events_1 = require("events");
var moves_1 = require("./moves");
var logger_1 = require("./logger");
var shapes_1 = require("./shapes");
var logger = (0, logger_1.default)("turn.js");
var NoCurrentPlayerError = (0, errors_1.createError)("NoCurrentPlayerError");
var NoNextPlayerError = (0, errors_1.createError)("NoNextPlayerError");
var PlayersNotEnoughError = (0, errors_1.createError)("PlayersNotEnoughError");
var InvalidArgumentError = (0, errors_1.createError)("InvalidArgumentError");
var InappropriateMoveError = (0, errors_1.createError)("InappropriateMoveError");
var InvalidArgumentTypeError = (0, errors_1.createTypeError)("InvalidArgumentTypeError");
/**
 * @event turn:switch
 * @event turn:holdon
 * @event turn:pick-two
 * @event turn:pick-three
 * @event turn:suspension
 */
var Turn = /** @class */ (function () {
    function Turn(props) {
        var _this = this;
        this.players = [];
        this.next = function () { return _this.players.find(function (player) { return player.turn; }); };
        this.all = function (fn) {
            if (fn && typeof fn === "function")
                _this.players.forEach(fn);
            return _this.players.length;
        };
        /**
         * @param {Number} skip ignore this number of players
         */
        this.switch = function (skip) {
            if (skip === void 0) { skip = 0; }
            var nextPlayerIndex = (_this.players.findIndex(function (player) { return player.turn; }) + ++skip) %
                _this.players.length;
            var nextPlayer = _this.players[nextPlayerIndex];
            var currentPlayer = _this.next();
            if (!currentPlayer) {
                throw NoCurrentPlayerError("");
            }
            currentPlayer.turn = false;
            nextPlayer.turn = true;
            _this.emitter.emit("turn:switch", skip, nextPlayer);
            return _this;
        };
        /**
         * @param {number} noOfPlayers how many players are to pick
         * @param {number} count how many cards is each player to pick
         *
         * @returns affected players
         */
        this.setToPick = function (noOfPlayers, count, noSwitch) {
            if (!Number(noOfPlayers))
                throw InvalidArgumentError("noOfPlayers");
            if (!Number(count))
                throw InvalidArgumentError("count");
            var currentPlayerIndex = noSwitch
                ? -1
                : _this.players.findIndex(function (player) { return player.turn; });
            var ret = [];
            for (var i = 1; i <= noOfPlayers; i++) {
                var nextPlayerIndex = ++currentPlayerIndex % _this.players.length;
                _this.players[nextPlayerIndex].toPick = count;
                ret.push(_this.players[nextPlayerIndex]);
            }
            return ret;
        };
        this.holdon = function (noSwitch) {
            _this.emitter.emit("turn:holdon", _this.skipped(_this.count() - 1));
            if (!noSwitch)
                _this.switch(_this.count() - 1);
        };
        this.pickTwo = function (noSwitch) {
            var nextPlayer = _this.next();
            if (!nextPlayer) {
                throw NoNextPlayerError("");
            }
            if (nextPlayer.toPick === 0 || nextPlayer.toPick === 2) {
                var affectedPlayers = _this.setToPick(1, nextPlayer.toPick + 2, noSwitch);
                if (!noSwitch)
                    nextPlayer.toPick = 0;
                _this.emitter.emit("turn:pick-two", affectedPlayers[0]);
                if (!noSwitch)
                    _this.switch();
                return _this;
            }
            else
                throw InappropriateMoveError("pickTwo");
        };
        this.pickThree = function (noSwitch) {
            var nextPlayer = _this.next();
            if (!nextPlayer) {
                throw NoNextPlayerError("");
            }
            if (nextPlayer.toPick === 0 || nextPlayer.toPick === 3) {
                var affectedPlayers = _this.setToPick(1, nextPlayer.toPick + 3, noSwitch);
                if (!noSwitch)
                    nextPlayer.toPick = 0;
                _this.emitter.emit("turn:pick-three", affectedPlayers[0]);
                if (!noSwitch)
                    _this.switch();
                return _this;
            }
            else
                throw InappropriateMoveError("pickThree");
        };
        this.skipped = function (no) {
            var ret = [];
            var currentPlayerIndex = _this.players.findIndex(function (player) { return player.turn; });
            for (var i = 1; i <= no; i++) {
                var nextPlayerIndex = ++currentPlayerIndex % _this.players.length;
                ret.push(_this.players[nextPlayerIndex]);
            }
            return ret;
        };
        /**
         * @param {Boolean} isStar check if the card played is a star
         */
        this.suspension = function (isStar, noSwitch) {
            _this.emitter.emit("turn:suspension", _this.skipped(isStar ? 2 : 1));
            if (!noSwitch)
                _this.switch(isStar ? 2 : 1);
        };
        this.generalMarket = function (noSwitch) {
            var affectedPlayers = _this.setToPick(_this.count() - 1, 1, noSwitch);
            _this.emitter.emit("turn:general-market", affectedPlayers);
            if (!noSwitch)
                _this.switch();
            return affectedPlayers;
        };
        this.count = function () { return _this.players.length; };
        /**
         * @param {Card} card
         */
        this.execute = function (card, noSwitch) {
            if (card.move === moves_1.default.GeneralMarket) {
                _this.generalMarket(noSwitch);
            }
            else if (card.move === moves_1.default.HoldOn) {
                _this.holdon(noSwitch);
            }
            else if (card.move === moves_1.default.PickThree) {
                _this.pickThree(noSwitch);
            }
            else if (card.move === moves_1.default.PickTwo) {
                _this.pickTwo(noSwitch);
            }
            else if (card.move === moves_1.default.Suspension) {
                _this.suspension(card.shape === shapes_1.default.Star, noSwitch);
            }
            else {
                if (!noSwitch)
                    _this.switch();
            }
        };
        if (!Array.isArray(props.players)) {
            throw InvalidArgumentTypeError("props.players", Array);
        }
        if (!props.emitter) {
            logger.warn("props.emitter not undefined");
            this.emitter = props.emitter = new events_1.EventEmitter();
        }
        else {
            this.emitter = props.emitter;
        }
        this.players = props.players;
        this.initPlayers();
    }
    Turn.prototype.initPlayers = function () {
        if (this.players.length === 0)
            throw PlayersNotEnoughError("players: ".concat(this.players.length));
        this.players.forEach(function (player, index) {
            player.turn = index === 0;
        });
    };
    return Turn;
}());
exports.Turn = Turn;
exports.default = Turn;
//# sourceMappingURL=turn.js.map